@@ -10,14 +10,14 @@
 %           CCCC   OOO   M   M  P      R  R   EEEEE   SSSSS  SSSSS            %
 %                                                                             %
 %                                                                             %
-%                  Image Compression/Decompression Methods                    %
+%             MagickCore Image Compression/Decompression Methods              %
 %                                                                             %
 %                           Software Design                                   %
 %                             John Cristy                                     %
 %                              May  1993                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,9 +41,11 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/color-private.h"
+#include "magick/cache.h"
 #include "magick/compress.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
@@ -74,7 +76,7 @@
 */
 struct _Ascii85Info
 {
-  long
+  ssize_t
     offset,
     line_break;
 
@@ -84,7 +86,7 @@ struct _Ascii85Info
 
 typedef struct HuffmanTable
 {
-  unsigned long
+  size_t
     id,
     code,
     length,
@@ -221,7 +223,7 @@ static const HuffmanTable
 %
 %  The format of the ASCII85Encode method is:
 %
-%      void Ascii85Encode(Image *image,const unsigned long code)
+%      void Ascii85Encode(Image *image,const size_t code)
 %
 %  A description of each parameter follows:
 %
@@ -238,16 +240,16 @@ static char *Ascii85Tuple(unsigned char *data)
   static char
     tuple[6];
 
-  register long
+  register ssize_t
     i,
     x;
 
-  unsigned long
+  size_t
     code,
     quantum;
 
-  code=((((unsigned long) data[0] << 8) | (unsigned long) data[1]) << 16) |
-    ((unsigned long) data[2] << 8) | (unsigned long) data[3];
+  code=((((size_t) data[0] << 8) | (size_t) data[1]) << 16) |
+    ((size_t) data[2] << 8) | (size_t) data[3];
   if (code == 0L)
     {
       tuple[0]='z';
@@ -257,7 +259,7 @@ static char *Ascii85Tuple(unsigned char *data)
   quantum=85UL*85UL*85UL*85UL;
   for (i=0; i < 4; i++)
   {
-    x=(long) (code/quantum);
+    x=(ssize_t) (code/quantum);
     code-=quantum*x;
     tuple[i]=(char) (x+(int) '!');
     quantum/=85L;
@@ -307,15 +309,15 @@ MagickExport void Ascii85Flush(Image *image)
 
 MagickExport void Ascii85Encode(Image *image,const unsigned char code)
 {
-  long
-    n;
-
   register char
     *q;
 
   register unsigned char
     *p;
 
+  ssize_t
+    n;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   assert(image->ascii85 != (Ascii85Info *) NULL);
@@ -409,15 +411,21 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
       mask=0x80;  \
     }  \
   runlength++;  \
-  bit=(unsigned long) ((byte & mask) != 0 ? 0x01 : 0x00); \
+  bit=(size_t) ((byte & mask) != 0 ? 0x01 : 0x00); \
   mask>>=1;  \
   if (bit != 0)  \
     runlength=0;  \
 }
 
+  CacheView
+    *image_view;
+
   const HuffmanTable
     *entry;
 
+  ExceptionInfo
+    *exception;
+
   HuffmanTable
     **mb_hash,
     **mw_hash;
@@ -428,27 +436,29 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
   int
     byte;
 
-  long
-    y;
-
   MagickBooleanType
     proceed;
 
   register IndexPacket
     *indexes;
 
-  register long
-    i,
-    x;
-
-  register PixelPacket
-    *q;
+  register ssize_t
+    i;
 
   register unsigned char
     *p;
 
+  size_t
+    bit,
+    code,
+    mask,
+    length,
+    null_lines,
+    runlength;
+
   ssize_t
-    count;
+    count,
+    y;
 
   unsigned char
     *scanline;
@@ -457,14 +467,6 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
     bail,
     color;
 
-  unsigned long
-    bit,
-    code,
-    mask,
-    length,
-    null_lines,
-    runlength;
-
   /*
     Allocate buffers.
   */
@@ -508,13 +510,21 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
   image->x_resolution=204.0;
   image->y_resolution=196.0;
   image->units=PixelsPerInchResolution;
-  for (y=0; ((y < (long) image->rows) && (null_lines < 3)); )
+  exception=(&image->exception);
+  image_view=AcquireAuthenticCacheView(image,exception);
+  for (y=0; ((y < (ssize_t) image->rows) && (null_lines < 3)); )
   {
+    register PixelPacket
+      *restrict q;
+
+    register ssize_t
+      x;
+
     /*
       Initialize scanline to white.
     */
     p=scanline;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
       *p++=(unsigned char) 0;
     /*
       Decode Huffman encoded scanline.
@@ -529,7 +539,7 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
     {
       if (byte == EOF)
         break;
-      if (x >= (long) image->columns)
+      if (x >= (ssize_t) image->columns)
         {
           while (runlength < 11)
             InputBit(bit);
@@ -553,7 +563,7 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
                 break;
               }
           }
-        code=(code << 1)+(unsigned long) bit;
+        code=(code << 1)+(size_t) bit;
         length++;
       } while (code == 0);
       if (bail != MagickFalse)
@@ -586,8 +596,8 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
         case TWId:
         case TBId:
         {
-          count+=entry->count;
-          if ((x+count) > (long) image->columns)
+          count+=(ssize_t) entry->count;
+          if ((x+count) > (ssize_t) image->columns)
             count=(ssize_t) image->columns-x;
           if (count > 0)
             {
@@ -608,7 +618,7 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
         case MBId:
         case EXId:
         {
-          count+=entry->count;
+          count+=(ssize_t) entry->count;
           break;
         }
         default:
@@ -621,24 +631,25 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
       Transfer scanline to image pixels.
     */
     p=scanline;
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
-    for (x=0; x < (long) image->columns; x++)
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       index=(IndexPacket) (*p++);
-      indexes[x]=index;
-      *q++=image->colormap[(long) index];
+      SetPixelIndex(indexes+x,index);
+      SetPixelRGBO(q,image->colormap+(ssize_t) index);
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       break;
     proceed=SetImageProgress(image,LoadImageTag,y,image->rows);
     if (proceed == MagickFalse)
       break;
     y++;
   }
-  image->rows=(unsigned long) MagickMax((size_t) y-3,1);
+  image_view=DestroyCacheView(image_view);
+  image->rows=(size_t) MagickMax((size_t) y-3,1);
   image->compression=FaxCompression;
   /*
     Free decoder memory.
@@ -665,7 +676,7 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
 %  The format of the HuffmanEncodeImage method is:
 %
 %      MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
-%        Image *image)
+%        Image *image,Image *inject_image)
 %
 %  A description of each parameter follows:
 %
@@ -673,13 +684,15 @@ MagickExport MagickBooleanType HuffmanDecodeImage(Image *image)
 %
 %    o image: the image.
 %
+%    o inject_image: inject into the image stream.
+%
 */
 MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
-  Image *image)
+  Image *image,Image *inject_image)
 {
 #define HuffmanOutputCode(entry)  \
 {  \
-  mask=1 << (entry->length-1);  \
+  mask=one << (entry->length-1);  \
   while (mask != 0)  \
   {  \
     OutputBit(((entry->code & mask) != 0 ? 1 : 0));  \
@@ -706,21 +719,20 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
   const HuffmanTable
     *entry;
 
+  ExceptionInfo
+    *exception;
+
   int
     k,
     runlength;
 
-  long
-    n,
-    y;
-
   Image
     *huffman_image;
 
   MagickBooleanType
     proceed;
 
-  register long
+  register ssize_t
     i,
     x;
 
@@ -730,32 +742,42 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
   register unsigned char
     *q;
 
+  size_t
+    mask,
+    one,
+    width;
+
+  ssize_t
+    n,
+    y;
+
   unsigned char
     byte,
     bit,
     *scanline;
 
-  unsigned long
-    mask,
-    width;
-
   /*
     Allocate scanline buffer.
   */
+  assert(image_info != (ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  width=image->columns;
+  assert(inject_image != (Image *) NULL);
+  assert(inject_image->signature == MagickSignature);
+  one=1;
+  width=inject_image->columns;
   if (LocaleCompare(image_info->magick,"FAX") == 0)
-    width=(unsigned long) MagickMax(image->columns,1728);
+    width=(size_t) MagickMax(inject_image->columns,1728);
   scanline=(unsigned char *) AcquireQuantumMemory((size_t) width+1UL,
     sizeof(*scanline));
   if (scanline == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      image->filename);
+      inject_image->filename);
   (void) ResetMagickMemory(scanline,0,width*sizeof(*scanline));
-  huffman_image=CloneImage(image,0,0,MagickTrue,&image->exception);
+  huffman_image=CloneImage(inject_image,0,0,MagickTrue,&image->exception);
   if (huffman_image == (Image *) NULL)
     {
       scanline=(unsigned char *) RelinquishMagickMemory(scanline);
@@ -778,24 +800,24 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
   /*
     Compress to 1D Huffman pixels.
   */
+  exception=(&huffman_image->exception);
   q=scanline;
-  for (y=0; y < (long) huffman_image->rows; y++)
+  for (y=0; y < (ssize_t) huffman_image->rows; y++)
   {
-    p=AcquireImagePixels(huffman_image,0,y,huffman_image->columns,1,
-      &huffman_image->exception);
+    p=GetVirtualPixels(huffman_image,0,y,huffman_image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) huffman_image->columns; x++)
+    for (x=0; x < (ssize_t) huffman_image->columns; x++)
     {
-      *q++=(unsigned char) (PixelIntensity(p) >= ((MagickRealType)
-        QuantumRange/2.0) ? 0 : 1);
+      *q++=(unsigned char) (GetPixelIntensity(huffman_image,p) >=
+        ((MagickRealType) QuantumRange/2.0) ? 0 : 1);
       p++;
     }
     /*
       Huffman encode scanline.
     */
     q=scanline;
-    for (n=(long) width; n > 0; )
+    for (n=(ssize_t) width; n > 0; )
     {
       /*
         Output white run.
@@ -811,7 +833,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
             entry=MWTable+((runlength/64)-1);
           else
             entry=EXTable+(MagickMin((size_t) runlength,2560)-1792)/64;
-          runlength-=entry->count;
+          runlength-=(long) entry->count;
           HuffmanOutputCode(entry);
         }
       entry=TWTable+MagickMin((size_t) runlength,63);
@@ -831,7 +853,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
               entry=MBTable+((runlength/64)-1);
               if (runlength >= 1792)
                 entry=EXTable+(MagickMin((size_t) runlength,2560)-1792)/64;
-              runlength-=entry->count;
+              runlength-=(long) entry->count;
               HuffmanOutputCode(entry);
             }
           entry=TBTable+MagickMin((size_t) runlength,63);
@@ -870,7 +892,7 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
       if (LocaleCompare(image_info->magick,"FAX") == 0)
         (void) WriteBlobByte(image,byte);
       else
-        Ascii85Encode(image, byte);
+        Ascii85Encode(image,byte);
     }
   if (LocaleCompare(image_info->magick,"FAX") != 0)
     Ascii85Flush(image);
@@ -879,174 +901,6 @@ MagickExport MagickBooleanType HuffmanEncodeImage(const ImageInfo *image_info,
   return(MagickTrue);
 }
 
-#if defined(MAGICKCORE_TIFF_DELEGATE)
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   H u f f m a n 2 D E n c o d e I m a g e                                   %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  Huffman2DEncodeImage() compresses an image via two-dimensional
-%  Huffman-coding.
-%
-%  The format of the Huffman2DEncodeImage method is:
-%
-%      MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
-%        Image *image)
-%
-%  A description of each parameter follows:
-%
-%    o image_info: the image info..
-%
-%    o image: the image.
-%
-*/
-MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
-  Image *image)
-{
-  char
-    filename[MaxTextExtent];
-
-  FILE
-    *file;
-
-  Image
-    *huffman_image;
-
-  ImageInfo
-    *write_info;
-
-  int
-    unique_file;
-
-  MagickBooleanType
-    status;
-
-  register long
-    i;
-
-  ssize_t
-    count;
-
-  TIFF
-    *tiff;
-
-  uint16
-    fillorder;
-
-  uint32
-    *byte_count,
-    strip_size;
-
-  unsigned char
-    *buffer;
-
-  /*
-    Write image as CCITTFax4 TIFF image to a temporary file.
-  */
-  assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  huffman_image=CloneImage(image,0,0,MagickTrue,&image->exception);
-  if (huffman_image == (Image *) NULL)
-    return(MagickFalse);
-  DestroyBlob(huffman_image);
-  huffman_image->blob=CloneBlobInfo((BlobInfo *) NULL);
-  (void) SetImageType(huffman_image,BilevelType);
-  file=(FILE *) NULL;
-  unique_file=AcquireUniqueFileResource(filename);
-  if (unique_file != -1)
-    file=fdopen(unique_file,"wb"); 
-  if ((unique_file == -1) || (file == (FILE *) NULL))
-    {
-      (void) CopyMagickString(image->filename,filename,MaxTextExtent);
-      ThrowFileException(&image->exception,FileOpenError,
-        "UnableToCreateTemporaryFile",image->filename);
-      return(MagickFalse);
-    }
-  (void) FormatMagickString(huffman_image->filename,MaxTextExtent,"tiff:%s",
-    filename);
-  write_info=CloneImageInfo(image_info);
-  SetImageInfoFile(write_info,file);
-  write_info->compression=Group4Compression;
-  (void) SetImageOption(write_info,"quantum:polarity","min-is-white");
-  status=WriteImage(write_info,huffman_image);
-  write_info=DestroyImageInfo(write_info);
-  if (status == MagickFalse)
-    return(MagickFalse);
-  tiff=TIFFOpen(filename,"rb");
-  if (tiff == (TIFF *) NULL)
-    {
-      huffman_image=DestroyImage(huffman_image);
-      (void) fclose(file);
-      (void) RelinquishUniqueFileResource(filename);
-      ThrowFileException(&image->exception,FileOpenError,"UnableToOpenFile",
-        image_info->filename);
-      return(MagickFalse);
-    }
-  /*
-    Allocate raw strip buffer.
-  */
-  byte_count=0;
-  (void) TIFFGetField(tiff,TIFFTAG_STRIPBYTECOUNTS,&byte_count);
-  strip_size=byte_count[0];
-  for (i=1; i < (long) TIFFNumberOfStrips(tiff); i++)
-    if (byte_count[i] > strip_size)
-      strip_size=byte_count[i];
-  buffer=(unsigned char *) AcquireQuantumMemory((size_t) strip_size,
-    sizeof(*buffer));
-  if (buffer == (unsigned char *) NULL)
-    {
-      TIFFClose(tiff);
-      huffman_image=DestroyImage(huffman_image);
-      (void) fclose(file);
-      (void) RelinquishUniqueFileResource(filename);
-      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-        image_info->filename);
-    }
-  /*
-    Compress runlength encoded to 2D Huffman pixels.
-  */
-  fillorder=FILLORDER_LSB2MSB;
-  (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_FILLORDER,&fillorder);
-  for (i=0; i < (long) TIFFNumberOfStrips(tiff); i++)
-  {
-    count=(ssize_t) TIFFReadRawStrip(tiff,(uint32) i,buffer,(long)
-      byte_count[i]);
-    if (fillorder == FILLORDER_LSB2MSB)
-      TIFFReverseBits(buffer,(unsigned long) count);
-    (void) WriteBlob(image,count,buffer);
-  }
-  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
-  TIFFClose(tiff);
-  huffman_image=DestroyImage(huffman_image);
-  (void) fclose(file);
-  (void) RelinquishUniqueFileResource(filename);
-  return(MagickTrue);
-}
-#else
-MagickExport MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
-  Image *image)
-{
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  (void) ThrowMagickException(&image->exception,GetMagickModule(),
-    MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (TIFF)",
-    image->filename);
-  return(MagickFalse);
-}
-#endif
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -1095,28 +949,28 @@ MagickExport MagickBooleanType LZWEncodeImage(Image *image,const size_t length,
 
   typedef struct _TableType
   {
-    long
+    ssize_t
       prefix,
       suffix,
       next;
   } TableType;
 
-  long
-    index;
-
-  register long
+  register ssize_t
     i;
 
-  TableType
-    *table;
-
-  unsigned long
+  size_t
     accumulator,
     number_bits,
     code_width,
     last_code,
     next_index;
 
+  ssize_t
+    index;
+
+  TableType
+    *table;
+
   /*
     Allocate string table.
   */
@@ -1144,32 +998,32 @@ MagickExport MagickBooleanType LZWEncodeImage(Image *image,const size_t length,
   }
   next_index=LZWEod+1;
   code_width=9;
-  last_code=(unsigned long) pixels[0];
-  for (i=1; i < (long) length; i++)
+  last_code=(size_t) pixels[0];
+  for (i=1; i < (ssize_t) length; i++)
   {
     /*
       Find string.
     */
-    index=(long) last_code;
+    index=(ssize_t) last_code;
     while (index != -1)
-      if ((table[index].prefix != (long) last_code) ||
-          (table[index].suffix != (long) pixels[i]))
+      if ((table[index].prefix != (ssize_t) last_code) ||
+          (table[index].suffix != (ssize_t) pixels[i]))
         index=table[index].next;
       else
         {
-          last_code=(unsigned long) index;
+          last_code=(size_t) index;
           break;
         }
-    if (last_code != (unsigned long) index)
+    if (last_code != (size_t) index)
       {
         /*
           Add string.
         */
         OutputCode(last_code);
-        table[next_index].prefix=(long) last_code;
+        table[next_index].prefix=(ssize_t) last_code;
         table[next_index].suffix=(short) pixels[i];
         table[next_index].next=table[last_code].next;
-        table[last_code].next=(long) next_index;
+        table[last_code].next=(ssize_t) next_index;
         next_index++;
         /*
           Did we just move up to next bit width?
@@ -1194,7 +1048,7 @@ MagickExport MagickBooleanType LZWEncodeImage(Image *image,const size_t length,
                 code_width=9;
               }
             }
-          last_code=(unsigned long) pixels[i];
+          last_code=(size_t) pixels[i];
       }
   }
   /*
@@ -1244,7 +1098,7 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
   int
     count;
 
-  register long
+  register ssize_t
     i,
     j;
 
@@ -1263,8 +1117,7 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
   if (packbits == (unsigned char *) NULL)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  i=(long) length;
-  while (i != 0)
+  for (i=(ssize_t) length; i != 0; )
   {
     switch (i)
     {
@@ -1306,7 +1159,7 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
               Packed run.
             */
             count=3;
-            while (((long) count < i) && (*pixels == *(pixels+count)))
+            while (((ssize_t) count < i) && (*pixels == *(pixels+count)))
             {
               count++;
               if (count >= 127)
@@ -1327,12 +1180,12 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
         {
           packbits[count+1]=pixels[count];
           count++;
-          if (((long) count >= (i-3)) || (count >= 127))
+          if (((ssize_t) count >= (i-3)) || (count >= 127))
             break;
         }
         i-=count;
         *packbits=(unsigned char) (count-1);
-        for (j=0; j <= (long) count; j++)
+        for (j=0; j <= (ssize_t) count; j++)
           (void) WriteBlobByte(image,packbits[j]);
         pixels+=count;
         break;
@@ -1375,13 +1228,27 @@ MagickExport MagickBooleanType PackbitsEncodeImage(Image *image,
 %      pixels to compress.
 %
 */
+
+static voidpf AcquireZIPMemory(voidpf context,unsigned int items,
+  unsigned int size)
+{
+  (void) context;
+  return((voidpf) AcquireQuantumMemory(items,size));
+}
+
+static void RelinquishZIPMemory(voidpf context,voidpf memory)
+{
+  (void) context;
+  memory=RelinquishMagickMemory(memory);
+}
+
 MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
   unsigned char *pixels)
 {
   int
     status;
 
-  register long
+  register ssize_t
     i;
 
   size_t
@@ -1407,8 +1274,8 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
   stream.avail_in=(unsigned int) length;
   stream.next_out=compress_pixels;
   stream.avail_out=(unsigned int) compress_packets;
-  stream.zalloc=(alloc_func) NULL;
-  stream.zfree=(free_func) NULL;
+  stream.zalloc=AcquireZIPMemory;
+  stream.zfree=RelinquishZIPMemory;
   stream.opaque=(voidpf) NULL;
   status=deflateInit(&stream,(int) (image->quality ==
     UndefinedCompressionQuality ? 7 : MagickMin(image->quality/10,9)));
@@ -1424,7 +1291,7 @@ MagickExport MagickBooleanType ZLIBEncodeImage(Image *image,const size_t length,
   if (status != Z_OK)
     ThrowBinaryException(CoderError,"UnableToZipCompressImage",image->filename)
   else
-    for (i=0; i < (long) compress_packets; i++)
+    for (i=0; i < (ssize_t) compress_packets; i++)
       (void) WriteBlobByte(image,compress_pixels[i]);
   compress_pixels=(unsigned char *) RelinquishMagickMemory(compress_pixels);
   return(status == Z_OK ? MagickTrue : MagickFalse);
